require './lib/praetor/service'

def run_service(klass, r)
  redis = Redis.new
  redis.del klass.endpoint
  redis.lpush klass.endpoint, JSON.generate(r)
  redis.lpush klass.endpoint, 'stop'
  klass.new.run
  _, response = redis.brpop r['key']
  JSON.parse response
end

RSpec.describe Praetor::Service do
  class SomeController
    def show
      'Hi there'
    end
  end

  class SomeService < Praetor::Service
    controller SomeController
  end

  let(:service) { SomeService.new }
  let(:req) do
    {
      'controller' => 'SomeController',
      'action' => 'show'
    }
  end

  context '#dispatch!' do
    it 'dispatches messages to the correct controller' do
      result = service.dispatch! req
      expect(result).to eq 'Hi there'
    end

    it "raises an error if the controller doesn't exist" do
      r = req.merge('controller' => 'NilController')
      expect { service.dispatch! r }.to raise_error
    end

    it "raises an error if the action doesn't exist" do
      r = req.merge('action' => 'create')
      expect { service.dispatch! r }.to raise_error
    end
  end

  context '#run' do
    it 'grabs the first request waiting on the queue' do
      r = req.merge 'key' => Rubyflake.generate.to_s(32)
      response = run_service SomeService, r
      expect(response['data']).to eq 'Hi there'
    end

    it 'returns responses with errors' do
      r = req.merge 'key' => Rubyflake.generate.to_s(32), 'controller' => 'NilController'
      response = run_service SomeService, r
      expect(response['status']).to eq 'error'
    end
  end
end
